#include <bcm2835.h>
#include <stdio.h>
#include "ssd1362.h"
#define WIDTH 160
#define HEIGHT 64


#define RST 25
#define DC  24


void er_oled_command(char cmd) {
    bcm2835_gpio_write(DC, LOW);
    bcm2835_spi_transfer(cmd);
}

void er_oled_data(char dat) {
    bcm2835_gpio_write(DC, HIGH);
    bcm2835_spi_transfer(dat);
}

void er_oled_begin()
{
    bcm2835_gpio_fsel(RST, BCM2835_GPIO_FSEL_OUTP);
    bcm2835_gpio_fsel(DC, BCM2835_GPIO_FSEL_OUTP);

    bcm2835_spi_begin();
    bcm2835_spi_setBitOrder(BCM2835_SPI_BIT_ORDER_MSBFIRST);     //The default
    bcm2835_spi_setDataMode(BCM2835_SPI_MODE0);                  //The default
    bcm2835_spi_setClockDivider(BCM2835_SPI_CLOCK_DIVIDER_2048);  //The default
    bcm2835_spi_chipSelect(BCM2835_SPI_CS0);                     //The default
    bcm2835_spi_setChipSelectPolarity(BCM2835_SPI_CS0, LOW);     //the default

    bcm2835_gpio_write(RST, HIGH);
    bcm2835_delay(10);
    bcm2835_gpio_write(RST, LOW);
    bcm2835_delay(10);
    bcm2835_gpio_write(RST, HIGH);

 
    er_oled_command(0xAE);  //Set Display Off 
    er_oled_command(0x15);  // Set Column Address 
    er_oled_command(0x00);
    er_oled_command(0x4F);
    er_oled_command(0x75);  // Set Row Address 
    er_oled_command(0x00);
    er_oled_command(0x3F);
    er_oled_command(0x81);  // Set Contrast Control 
    er_oled_command(0xB0);
    er_oled_command(0xA0);  // Set Re-map
    er_oled_command(0xC1);
    er_oled_command(0xA1);  // Set Display Start Line
    er_oled_command(0x00);
    er_oled_command(0xA2);  // Set Display Offset
    er_oled_command(0x00);
    er_oled_command(0xA3);  // Set Vertical Scroll Area
    er_oled_command(0x00); 
    er_oled_command(0x40);
    er_oled_command(0xA4);  // Set Display Mode (Normal display)
    er_oled_command(0xA8);  // Set MUX Ratio 
    er_oled_command(0x3F);
    er_oled_command(0xAB);  // Function Selection A
    er_oled_command(0x01);  // Select external VDD
    er_oled_command(0xAD);  // External /Internal IREF Selection
    er_oled_command(0x8E);  // Select external IREF
    er_oled_command(0xB1);  // Set Phase Length
    er_oled_command(0x82);
    er_oled_command(0xB3);  // Set Front Clock Divider/Oscillator Frequency
    er_oled_command(0xA0);
    er_oled_command(0xB6);  // Set Second precharge Period
    er_oled_command(0x04);
    er_oled_command(0xB9);  // Linear LUT
    er_oled_command(0xBC);  // Set Pre-charge voltage 
    er_oled_command(0x04);
    er_oled_command(0xBD);  // Pre-charge voltage capacitor Selection
    er_oled_command(0x01);
    er_oled_command(0xBE);  // Set VCOMH 
    er_oled_command(0x05);  // 0.82*VCC
    er_oled_command(0xAF);  // Set Display ON
   
}


void er_oled_SetWindow(uint8_t Xstart, uint8_t Ystart, uint8_t Xend, uint8_t Yend)
{ 
  er_oled_command(0x15);
  er_oled_command(Xstart);
  er_oled_command(Xend);
  er_oled_command(0x75);
  er_oled_command(Ystart);
  er_oled_command(Yend);
 
}

void er_oled_clear()
{int i,row;
er_oled_SetWindow(0, 0, WIDTH/2-1, HEIGHT-1);
  for (row = 0; row < HEIGHT; row++) {              
        for(i = 0; i< WIDTH/2; i++ ) {
          er_oled_data(0x00);// write data       
        }        
  }

}

void er_oled_char(uint8_t x, uint8_t y,  char  acsii, uint8_t mode)
{ uint8_t i, str ;uint16_t OffSet;
  x=x/2;
  OffSet = (acsii - 32)*16;
  er_oled_SetWindow(x, y, x+3, y+15);
  for (i=0;i<16;i++)
  {     str =AsciiLib[OffSet + i];  
        if(mode) str=~str;
         er_oled_Data_processing (str);             
  }
}



void er_oled_string(uint8_t x, uint8_t y, char *pString,  uint8_t Mode)
{     
  while(1)
  {
        if (*pString == 0)
        {
            return;
        }
            er_oled_char(x, y, *pString,Mode);
            x += 8;
            pString += 1;              
  }   
}

void er_oled_Data_processing(uint8_t temp)  //turns 1byte B/W data to 4 bye gray data  with 8 Pixel
{uint8_t temp1,temp2;

  if(temp&0x80)temp1=0xf0;
  else temp1=0x00;
  if(temp&0x40)temp2=0x0f;
  else temp2=0x00;
  temp1=temp1|temp2;
  er_oled_data(temp1); //Pixel1,Pixel2
  if(temp&0x20)temp1=0xf0;
  else temp1=0x00;
  if(temp&0x10)temp2=0x0f;
  else temp2=0x00;
  temp1=temp1|temp2;
  er_oled_data(temp1);  //Pixel3,Pixel4                
  if(temp&0x08)temp1=0xf0;
  else temp1=0x00;
  if(temp&0x04)temp2=0x0f;
  else temp2=0x00;
  temp1=temp1|temp2;
  er_oled_data(temp1);  //Pixel5,Pixel6               
  if(temp&0x02)temp1=0xf0;
  else temp1=0x00;
  if(temp&0x01)temp2=0x0f;
  else temp2=0x00;
  temp1=temp1|temp2;
  er_oled_data(temp1);  //Pixel7,Pixel8               
}

void er_oled_bitmap_mono(const uint8_t  * pBuf,uint8_t mode)
{ uint8_t row,col; 
  er_oled_SetWindow(0, 0, WIDTH/2-1, HEIGHT-1);
  for (row = 0; row < HEIGHT; row++) {              
        for(col = 0;col<WIDTH/8; col++ ) {  
	if(mode)er_oled_Data_processing(~pBuf[col+row*(WIDTH/8)]);      
       else er_oled_Data_processing(pBuf[col+row*(WIDTH/8)]);              
        }
  }    	
}

void er_oled_bitmap_gray(const uint8_t  * pBuf,uint8_t mode)
{   uint8_t row,col; 
  er_oled_SetWindow(0, 0, WIDTH/2-1, HEIGHT-1);
  for (row = 0; row < HEIGHT; row++) {              
        for(col = 0;col<WIDTH/2; col++ ) {
	if(mode)er_oled_data(~pBuf[col+row*(WIDTH/2)]); 
        else er_oled_data(pBuf[col+row*(WIDTH/2)]);     
        }
  }    	
}